@@ -13,10 +13,6 @@
 /* INTERFACE                          */
 /**************************************/
 
-/*      define ENABLE_BOUNDS_CHECKING here to do bounds checking! */
-
-#define ENABLE_BOUNDS_CHECKING
-
 /*      automatic self-configuring routine: */
 
 unit    Set_Auto_config(void);                       /* 0 = ok, 1..5 = error */
@@ -34,6 +30,10 @@ unitptr Set_Resize (unitptr oldaddr, N_int elements);       /* realloc       */
 void    Set_Empty  (unitptr addr);                          /* X = {}        */
 void    Set_Fill   (unitptr addr);                          /* X = ~{}       */
 
+void    Set_Empty_Interval(unitptr addr, N_int lower, N_int upper);
+void    Set_Fill_Interval (unitptr addr, N_int lower, N_int upper);
+void    Set_Flip_Interval (unitptr addr, N_int lower, N_int upper);
+
 /*      set operations on elements: */
 
 void    Set_Insert (unitptr addr, N_int index);             /* X = X + {x}   */
@@ -169,7 +169,7 @@ void    Set_Copy        (unitptr X, unitptr Y);             /* X = Y         */
 /**************************************/
 /* CREATED      01.11.93              */
 /**************************************/
-/* MODIFIED     20.01.97              */
+/* MODIFIED     01.02.97              */
 /**************************************/
 /* COPYRIGHT    Steffen Beyer         */
 /**************************************/
